function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import PropTypes from 'prop-types';
import { camelize, normalizeLocation, isRouteChanged, getHostRouterView } from './util';

function guardEvent(e) {
  // don't redirect with control keys
  if (e.metaKey || e.altKey || e.ctrlKey || e.shiftKey) return; // don't redirect when preventDefault called

  if (e.defaultPrevented) return; // don't redirect on right click

  if (e.button !== undefined && e.button !== 0) return; // don't redirect if `target="_blank"`

  if (e.currentTarget && e.currentTarget.getAttribute) {
    const target = e.currentTarget.getAttribute('target');
    if (/\b_blank\b/i.test(target)) return;
  } // this may be a Weex event which doesn't have this method


  if (e.preventDefault) {
    e.preventDefault();
  }

  return true;
}

let routerLinkSeed = 0;

class RouterLink extends React.Component {
  constructor(props) {
    super(props);

    _defineProperty(this, "unplugin", void 0);

    const router = props.router;
    this.state = {
      inited: false,
      router,
      currentRoute: router ? router.currentRoute : null,
      parentRoute: null,
      seed: routerLinkSeed++
    };
  }

  _remount() {
    let {
      router,
      seed
    } = this.state;

    if (this.unplugin) {
      this.unplugin();
      this.unplugin = undefined;
    }

    let routerView = getHostRouterView(this);
    if (!router && routerView) router = routerView.state.router;
    this.unplugin = router ? router.plugin({
      name: `router-link-plugin-${seed}`,
      onRouteChange: currentRoute => {
        this.setState({
          currentRoute
        });
        if (this.props.onRouteChange) this.props.onRouteChange(currentRoute);
      }
    }) : undefined;
    this.setState({
      inited: true,
      router,
      parentRoute: routerView ? routerView.state.currentRoute : null
    });
  }

  componentDidMount() {
    this._remount();
  }

  componentWillUnmount() {
    if (this.unplugin) {
      this.unplugin();
      this.unplugin = undefined;
    }
  }

  shouldComponentUpdate(nextProps, nextState) {
    if (this.props.router !== nextProps.router) return true;
    if (this.props.to !== nextProps.to) return true;
    if (this.props.replace !== nextProps.replace) return true;
    if (this.props.tag !== nextProps.tag) return true;
    if (this.props.activeClass !== nextProps.activeClass) return true;
    if (this.props.exact !== nextProps.exact) return true;
    if (this.props.exactActiveClass !== nextProps.exactActiveClass) return true;
    if (this.props.event !== nextProps.event) return true;
    if (this.state.inited !== nextState.inited) return true;
    if (isRouteChanged(this.state.parentRoute, nextState.parentRoute)) return true;
    if (isRouteChanged(this.state.currentRoute, nextState.currentRoute)) return true;
    return false;
  }

  componentDidUpdate(prevProps) {
    if (this.props.router !== prevProps.router) {
      this.setState({
        router: this.props.router
      }, () => this._remount());
    }
  }

  render() {
    if (!this.state.inited) return null;

    let _this$props = this.props,
        {
      // eslint-disable-next-line no-unused-vars
      router: router1,
      tag,
      to,
      exact,
      replace,
      append,
      event,
      children = [],
      activeClass,
      exactActiveClass
    } = _this$props,
        remainProps = _objectWithoutProperties(_this$props, ["router", "tag", "to", "exact", "replace", "append", "event", "children", "activeClass", "exactActiveClass"]);

    const {
      currentRoute: current = {
        path: ''
      },
      router
    } = this.state;
    to = normalizeLocation(to, this.state.parentRoute, {
      append
    });

    if (router && router.linkActiveClass) {
      activeClass = activeClass ? `${activeClass} ${router.linkActiveClass}` : router.linkActiveClass;
    }

    if (router && router.linkExactActiveClass) {
      exactActiveClass = exactActiveClass ? `${exactActiveClass} ${router.linkExactActiveClass}` : router.linkExactActiveClass;
    }

    let fallbackClass = '';

    if (to && current) {
      fallbackClass = exact ? to.path === current.path ? exactActiveClass : '' : to && current.path.startsWith(to.path) ? activeClass : '';
    }

    if (fallbackClass) {
      if (remainProps.className) remainProps.className = `${fallbackClass} ${remainProps.className}`;else remainProps.className = fallbackClass;
    }

    if (!Array.isArray(event)) event = [event];
    const events = {};
    event.forEach(evt => {
      events[camelize(`on-${evt}`)] = e => {
        guardEvent(e);
        if (!to || !router || remainProps.disabled) return;
        if (replace) router.replace(to);else router.push(to);
      };
    });
    if (to && tag === 'a') remainProps.href = to.path;
    return React.createElement(tag, Object.assign(remainProps, events), ...children);
  }

}

_defineProperty(RouterLink, "propTypes", void 0);

_defineProperty(RouterLink, "defaultProps", void 0);

RouterLink.propTypes = {
  to: PropTypes.oneOfType([PropTypes.string, PropTypes.object]).isRequired,
  replace: PropTypes.bool,
  append: PropTypes.bool,
  tag: PropTypes.oneOfType([PropTypes.string, PropTypes.elementType]),
  activeClass: PropTypes.string,
  exact: PropTypes.bool,
  exactActiveClass: PropTypes.string,
  event: PropTypes.oneOfType([PropTypes.string, PropTypes.arrayOf(PropTypes.string)]),
  onRouteChange: PropTypes.func
};
RouterLink.defaultProps = {
  tag: 'a',
  activeClass: 'router-link-active',
  exactActiveClass: 'exact-active-class',
  event: 'click'
};
export { RouterLink };
export default function createRouterLink(router) {
  return React.forwardRef((props, ref) => React.createElement(RouterLink, _objectSpread(_objectSpread({
    router
  }, props), {}, {
    ref
  }), props.children));
}